@@ -56,7 +56,7 @@ class _TargetRefDBDumper(Dumper):
                     lp.write(target_entity_id=e._id, db_name=r.name,
                              db_name_other_details=r.other_details,
                              db_code=r.code, db_accession=r.accession,
-                             seq_db_isoform=r.isoform or ihm.unknown,
+                             seq_db_isoform=r.isoform,
                              seq_db_align_begin=db_begin,
                              seq_db_align_end=db_end,
                              ncbi_taxonomy_id=r.ncbi_taxonomy_id,
@@ -240,6 +240,14 @@ class _TemplateTransformDumper(Dumper):
                          **_get_transform(t.rot_matrix, t.tr_vector))
 
 
+def _get_target_asym(asym_or_segment):
+    """Get AsymUnit from something that could be AsymUnit or AsymUnitSegment"""
+    if isinstance(asym_or_segment, ihm.AsymUnitSegment):
+        return asym_or_segment.asym
+    else:
+        return asym_or_segment
+
+
 class _AlignmentDumper(Dumper):
     def finalize(self, system):
         for n, tmpl in enumerate(system.templates):
@@ -255,6 +263,7 @@ class _AlignmentDumper(Dumper):
         self.dump_template_details(system, writer)
         self.dump_template_poly(system, writer)
         self.dump_template_poly_segment(system, writer)
+        self.dump_template_non_poly(system, writer)
         self.dump_template_ref_db(system, writer)
         self.dump_target_template_poly_mapping(system, writer)
         self.dump_info(system, writer)
@@ -269,7 +278,8 @@ class _AlignmentDumper(Dumper):
                  "template_entity_type", "template_trans_matrix_id",
                  "template_data_id", "target_asym_id",
                  "template_label_asym_id",
-                 "template_label_entity_id", "template_model_num"]) as lp:
+                 "template_label_entity_id", "template_model_num",
+                 "template_auth_asym_id"]) as lp:
             for a in system.alignments:
                 for s in a.pairs:
                     # get Template from TemplateSegment
@@ -278,16 +288,18 @@ class _AlignmentDumper(Dumper):
                            else "customized")
                     poly = ("polymer" if tmpl.entity.is_polymeric()
                             else "non-polymer")
+                    tgt_asym = _get_target_asym(s.target)
                     lp.write(ordinal_id=next(ordinal),
                              template_id=tmpl._id,
                              template_origin=org,
                              template_entity_type=poly,
                              template_trans_matrix_id=tmpl.transformation._id,
                              template_data_id=tmpl._data_id,
-                             target_asym_id=s.target.asym._id,
+                             target_asym_id=tgt_asym._id,
                              template_label_asym_id=tmpl.asym_id,
                              template_label_entity_id=tmpl.entity._id,
-                             template_model_num=tmpl.model_num)
+                             template_model_num=tmpl.model_num,
+                             template_auth_asym_id=tmpl.strand_id)
 
     def _get_sequence(self, entity):
         """Get the sequence for an entity as a string"""
@@ -310,6 +322,8 @@ class _AlignmentDumper(Dumper):
                  "seq_one_letter_code_can"]) as lp:
             for tmpl in system.templates:
                 entity = tmpl.entity
+                if not entity.is_polymeric():
+                    continue
                 lp.write(template_id=tmpl._id,
                          seq_one_letter_code=self._get_sequence(entity),
                          seq_one_letter_code_can=self._get_canon(entity))
@@ -319,11 +333,24 @@ class _AlignmentDumper(Dumper):
                          ["id", "template_id", "residue_number_begin",
                           "residue_number_end"]) as lp:
             for s in system.template_segments:
+                if not s.template.entity.is_polymeric():
+                    continue
                 lp.write(
                     id=s._segment_id, template_id=s.template._id,
                     residue_number_begin=s.seq_id_range[0],
                     residue_number_end=s.seq_id_range[1])
 
+    def dump_template_non_poly(self, system, writer):
+        with writer.loop(
+                "_ma_template_non_poly",
+                ["template_id", "comp_id", "details"]) as lp:
+            for tmpl in system.templates:
+                entity = tmpl.entity
+                if entity.is_polymeric():
+                    continue
+                lp.write(template_id=tmpl._id, comp_id=entity.sequence[0].id,
+                         details=entity.description)
+
     def dump_template_ref_db(self, system, writer):
         with writer.loop(
                 "_ma_template_ref_db_details",
@@ -342,12 +369,18 @@ class _AlignmentDumper(Dumper):
                           "target_seq_id_begin", "target_seq_id_end"]) as lp:
             for a in system.alignments:
                 for p in a.pairs:
-                    lp.write(
-                        id=next(ordinal),
-                        template_segment_id=p.template._segment_id,
-                        target_asym_id=p.target.asym._id,
-                        target_seq_id_begin=p.target.seq_id_range[0],
-                        target_seq_id_end=p.target.seq_id_range[1])
+                    if isinstance(p.target, ihm.AsymUnitSegment):
+                        lp.write(
+                            id=next(ordinal),
+                            template_segment_id=p.template._segment_id,
+                            target_asym_id=p.target.asym._id,
+                            target_seq_id_begin=p.target.seq_id_range[0],
+                            target_seq_id_end=p.target.seq_id_range[1])
+                    elif p.target.entity.is_polymeric():
+                        lp.write(
+                            id=next(ordinal),
+                            template_segment_id=p.template._segment_id,
+                            target_asym_id=p.target._id)
 
     def dump_info(self, system, writer):
         with writer.loop(
@@ -357,7 +390,10 @@ class _AlignmentDumper(Dumper):
                  "alignment_mode"]) as lp:
             for a in system.alignments:
                 if a.pairs:
-                    align_len = max(len(s.gapped_sequence) for pair in a.pairs
+                    align_len = max(len(s.gapped_sequence
+                                        if hasattr(s, 'gapped_sequence')
+                                        else s.entity.sequence)
+                                    for pair in a.pairs
                                     for s in (pair.template, pair.target))
                 else:
                     align_len = None
@@ -380,11 +416,16 @@ class _AlignmentDumper(Dumper):
                  "sequence_identity_denominator_other_details"]) as lp:
             for a in system.alignments:
                 for s in a.pairs:
+                    # We can't populate template_segment_id for nonpolymers;
+                    # the other fields likely make no sense either
+                    if not s.template.entity.is_polymeric():
+                        continue
                     denom = s.identity.denominator
                     od = s.identity.other_details
+                    tgt_asym = _get_target_asym(s.target)
                     lp.write(ordinal_id=next(ordinal), alignment_id=a._id,
                              template_segment_id=s.template._segment_id,
-                             target_asym_id=s.target.asym._id,
+                             target_asym_id=tgt_asym._id,
                              score_type=s.score.type,
                              score_type_other_details=s.score.other_details,
                              score_value=s.score.value,
@@ -402,12 +443,14 @@ class _AlignmentDumper(Dumper):
                 # todo: don't duplicate sequences
                 for s in a.pairs:
                     # 1=target, 2=template
-                    lp.write(ordinal_id=next(ordinal), alignment_id=a._id,
-                             target_template_flag=1,
-                             sequence=s.target.gapped_sequence)
-                    lp.write(ordinal_id=next(ordinal), alignment_id=a._id,
-                             target_template_flag=2,
-                             sequence=s.template.gapped_sequence)
+                    if hasattr(s.target, 'gapped_sequence'):
+                        lp.write(ordinal_id=next(ordinal), alignment_id=a._id,
+                                 target_template_flag=1,
+                                 sequence=s.target.gapped_sequence)
+                    if hasattr(s.template, 'gapped_sequence'):
+                        lp.write(ordinal_id=next(ordinal), alignment_id=a._id,
+                                 target_template_flag=2,
+                                 sequence=s.template.gapped_sequence)
 
 
 class _ProtocolDumper(Dumper):
