@@ -435,7 +435,7 @@ C
             isoform='testiso', ncbi_taxonomy_id='1234',
             organism_scientific='testorg')
         ref2 = modelcif.reference.UniProt(code='c2', accession='a2')
-        ref3 = CustomRef(code='c3', accession='a3')
+        ref3 = CustomRef(code='c3', accession='a3', isoform=ihm.unknown)
 
         e1 = modelcif.Entity('ACGT', references=[ref1, ref2, ref3])
         e1._id = 1
@@ -457,7 +457,7 @@ _ma_target_ref_db_details.seq_db_align_end
 _ma_target_ref_db_details.ncbi_taxonomy_id
 _ma_target_ref_db_details.organism_scientific
 1 UNP . testcode testacc testiso 4 8 1234 testorg
-1 UNP . c2 a2 ? 1 4 . .
+1 UNP . c2 a2 . 1 4 . .
 1 Other 'my custom ref' c3 a3 ? 1 4 . .
 #
 """)
@@ -486,7 +486,8 @@ _ma_target_ref_db_details.organism_scientific
         tr = modelcif.Transformation.identity()
         tr._id = 42
         t = modelcif.Template(tmp_e, asym_id='H', model_num=1, name='testtmp',
-                              transformation=tr, references=[ref1, ref2])
+                              transformation=tr, references=[ref1, ref2],
+                              strand_id='Z')
         t._data_id = 99
         p = modelcif.alignment.Pair(
             template=t.segment('AC-G', 1, 3),
@@ -517,7 +518,8 @@ _ma_template_details.target_asym_id
 _ma_template_details.template_label_asym_id
 _ma_template_details.template_label_entity_id
 _ma_template_details.template_model_num
-1 1 'reference database' polymer 42 99 A H 1 1
+_ma_template_details.template_auth_asym_id
+1 1 'reference database' polymer 42 99 A H 1 1 Z
 #
 #
 loop_
@@ -586,6 +588,266 @@ _ma_alignment.sequence
 1 1 1 ACE-
 2 1 2 AC-G
 #
+""")
+
+    def test_alignment_no_segment(self):
+        """Test AlignmentDumper with no poly segments specified"""
+
+        class Alignment(modelcif.alignment.Global,
+                        modelcif.alignment.Pairwise):
+            pass
+
+        system = modelcif.System()
+        tmp_e = modelcif.Entity('ACG')
+        tmp_e._id = 1
+        tgt_e = modelcif.Entity('ACE')
+        tgt_e._id = 2
+        system.entities.extend((tmp_e, tgt_e))
+        asym = modelcif.AsymUnit(tgt_e, id='A')
+        asym._id = 'A'
+        system.asym_units.append(asym)
+        tr = modelcif.Transformation.identity()
+        tr._id = 42
+        t = modelcif.Template(tmp_e, asym_id='H', model_num=1, name='testtmp',
+                              transformation=tr, references=[])
+        t._data_id = 99
+        t._segment_id = 8
+        system.templates.append(t)
+        p = modelcif.alignment.Pair(
+            template=t, target=asym,
+            score=modelcif.alignment.BLASTEValue("1e-15"),
+            identity=modelcif.alignment.ShorterSequenceIdentity(42.))
+        aln = Alignment(name='testaln', pairs=[p])
+        aln._data_id = 100
+        system.alignments.append(aln)
+        dumper = modelcif.dumper._AlignmentDumper()
+        dumper.finalize(system)
+        out = _get_dumper_output(dumper, system)
+        self.assertEqual(out, """#
+loop_
+_ma_template_details.ordinal_id
+_ma_template_details.template_id
+_ma_template_details.template_origin
+_ma_template_details.template_entity_type
+_ma_template_details.template_trans_matrix_id
+_ma_template_details.template_data_id
+_ma_template_details.target_asym_id
+_ma_template_details.template_label_asym_id
+_ma_template_details.template_label_entity_id
+_ma_template_details.template_model_num
+_ma_template_details.template_auth_asym_id
+1 1 customized polymer 42 99 A H 1 1 H
+#
+#
+loop_
+_ma_template_poly.template_id
+_ma_template_poly.seq_one_letter_code
+_ma_template_poly.seq_one_letter_code_can
+1 ACG ACG
+#
+#
+loop_
+_ma_target_template_poly_mapping.id
+_ma_target_template_poly_mapping.template_segment_id
+_ma_target_template_poly_mapping.target_asym_id
+_ma_target_template_poly_mapping.target_seq_id_begin
+_ma_target_template_poly_mapping.target_seq_id_end
+1 8 A . .
+#
+#
+loop_
+_ma_alignment_info.alignment_id
+_ma_alignment_info.data_id
+_ma_alignment_info.software_group_id
+_ma_alignment_info.alignment_length
+_ma_alignment_info.alignment_type
+_ma_alignment_info.alignment_mode
+1 100 . 3 'target-template pairwise alignment' global
+#
+#
+loop_
+_ma_alignment_details.ordinal_id
+_ma_alignment_details.alignment_id
+_ma_alignment_details.template_segment_id
+_ma_alignment_details.target_asym_id
+_ma_alignment_details.score_type
+_ma_alignment_details.score_type_other_details
+_ma_alignment_details.score_value
+_ma_alignment_details.percent_sequence_identity
+_ma_alignment_details.sequence_identity_denominator
+_ma_alignment_details.sequence_identity_denominator_other_details
+1 1 8 A 'BLAST e-value' . 1e-15 42.000 'Length of the shorter sequence' .
+#
+""")
+
+    def test_alignment_non_poly_with_alignment(self):
+        """Test AlignmentDumper with nonpolymeric template, with alignment"""
+
+        class Alignment(modelcif.alignment.Global,
+                        modelcif.alignment.Pairwise):
+            pass
+
+        system = modelcif.System()
+        e1 = ihm.Entity([ihm.NonPolymerChemComp('HEM')], description='heme')
+        e1._id = 1
+        system.entities.append(e1)
+        asym = modelcif.AsymUnit(e1)
+        asym._id = 'A'
+        system.asym_units.append(asym)
+        tr = modelcif.Transformation.identity()
+        tr._id = 42
+        t = modelcif.Template(e1, asym_id='H', model_num=1, name='testtmp',
+                              transformation=tr, references=[])
+        t._data_id = 99
+        system.templates.append(t)
+        p = modelcif.alignment.Pair(
+            template=t, target=asym, score=None, identity=None)
+        aln = Alignment(name='testaln', pairs=[p])
+        aln._data_id = 100
+        system.alignments.append(aln)
+        dumper = modelcif.dumper._AlignmentDumper()
+        dumper.finalize(system)
+        out = _get_dumper_output(dumper, system)
+        self.assertEqual(out, """#
+loop_
+_ma_template_details.ordinal_id
+_ma_template_details.template_id
+_ma_template_details.template_origin
+_ma_template_details.template_entity_type
+_ma_template_details.template_trans_matrix_id
+_ma_template_details.template_data_id
+_ma_template_details.target_asym_id
+_ma_template_details.template_label_asym_id
+_ma_template_details.template_label_entity_id
+_ma_template_details.template_model_num
+_ma_template_details.template_auth_asym_id
+1 1 customized non-polymer 42 99 A H 1 1 H
+#
+#
+loop_
+_ma_template_non_poly.template_id
+_ma_template_non_poly.comp_id
+_ma_template_non_poly.details
+1 HEM heme
+#
+#
+loop_
+_ma_alignment_info.alignment_id
+_ma_alignment_info.data_id
+_ma_alignment_info.software_group_id
+_ma_alignment_info.alignment_length
+_ma_alignment_info.alignment_type
+_ma_alignment_info.alignment_mode
+1 100 . 1 'target-template pairwise alignment' global
+#
+""")
+
+    def test_alignment_non_poly_with_poly_alignment(self):
+        """Test AlignmentDumper with nonpolymeric template, poly alignment"""
+
+        class Alignment(modelcif.alignment.Global,
+                        modelcif.alignment.Pairwise):
+            pass
+
+        system = modelcif.System()
+        e1 = ihm.Entity([ihm.NonPolymerChemComp('HEM')], description='heme')
+        e1._id = 1
+        system.entities.append(e1)
+        asym = modelcif.AsymUnit(e1)
+        asym._id = 'A'
+        system.asym_units.append(asym)
+        tr = modelcif.Transformation.identity()
+        tr._id = 42
+        t = modelcif.Template(e1, asym_id='H', model_num=1, name='testtmp',
+                              transformation=tr, references=[])
+        t._data_id = 99
+        system.templates.append(t)
+        # A TemplateSegment doesn't make a lot of sense for a nonpoly template,
+        # but allow it; just don't write out ma_template_poly_segment for it
+        p = modelcif.alignment.Pair(
+            template=t.segment('X', 1, 1), target=asym,
+            score=None, identity=None)
+        aln = Alignment(name='testaln', pairs=[p])
+        aln._data_id = 100
+        system.alignments.append(aln)
+        dumper = modelcif.dumper._AlignmentDumper()
+        system._before_write()  # populate system.template_segments
+        dumper.finalize(system)
+        out = _get_dumper_output(dumper, system)
+        self.assertEqual(out, """#
+loop_
+_ma_template_details.ordinal_id
+_ma_template_details.template_id
+_ma_template_details.template_origin
+_ma_template_details.template_entity_type
+_ma_template_details.template_trans_matrix_id
+_ma_template_details.template_data_id
+_ma_template_details.target_asym_id
+_ma_template_details.template_label_asym_id
+_ma_template_details.template_label_entity_id
+_ma_template_details.template_model_num
+_ma_template_details.template_auth_asym_id
+1 1 customized non-polymer 42 99 A H 1 1 H
+#
+#
+loop_
+_ma_template_non_poly.template_id
+_ma_template_non_poly.comp_id
+_ma_template_non_poly.details
+1 HEM heme
+#
+#
+loop_
+_ma_alignment_info.alignment_id
+_ma_alignment_info.data_id
+_ma_alignment_info.software_group_id
+_ma_alignment_info.alignment_length
+_ma_alignment_info.alignment_type
+_ma_alignment_info.alignment_mode
+1 100 . 1 'target-template pairwise alignment' global
+#
+#
+loop_
+_ma_alignment.ordinal_id
+_ma_alignment.alignment_id
+_ma_alignment.target_template_flag
+_ma_alignment.sequence
+1 1 2 X
+#
+""")
+
+    def test_non_poly_template_no_aln(self):
+        """Test AlignmentDumper with nonpolymeric template, no alignments"""
+        system = modelcif.System()
+        # Polymeric entity
+        e1 = ihm.Entity('ACGT')
+        t1 = modelcif.Template(
+            e1, asym_id="A", model_num=1, name="test template",
+            transformation=modelcif.Transformation.identity())
+        t1._id = 1
+        # Non-polymeric entity
+        e2 = ihm.Entity([ihm.NonPolymerChemComp('HEM')], description='heme')
+        t2 = modelcif.Template(
+            e2, asym_id="B", model_num=1, name="test template",
+            transformation=modelcif.Transformation.identity())
+        t2._id = 2
+        system.templates.extend((t1, t2))
+        dumper = modelcif.dumper._AlignmentDumper()
+        out = _get_dumper_output(dumper, system)
+        self.assertEqual(out, """#
+loop_
+_ma_template_poly.template_id
+_ma_template_poly.seq_one_letter_code
+_ma_template_poly.seq_one_letter_code_can
+1 ACGT ACGT
+#
+#
+loop_
+_ma_template_non_poly.template_id
+_ma_template_non_poly.comp_id
+_ma_template_non_poly.details
+2 HEM heme
+#
 """)
 
     def test_template_transform_dumper(self):
