@@ -214,13 +214,22 @@ _ma_template_details.target_asym_id
 _ma_template_details.template_label_asym_id
 _ma_template_details.template_label_entity_id
 _ma_template_details.template_model_num
-1 1 'reference database' polymer 1 2 A B 3 4
+_ma_template_details.template_auth_asym_id
+1 1 'reference database' polymer 1 2 A B 3 4 Z
 """
         s, = modelcif.reader.read(StringIO(cif))
         t, = s.templates
         self.assertEqual(t.entity._id, '3')
         self.assertEqual(t.model_num, 4)
         self.assertEqual(t.asym_id, 'B')
+        self.assertEqual(t.strand_id, 'Z')
+        # An 'alignment' should have been created for the implied
+        # template - target_asym_id correspondence
+        a, = s.alignments
+        p, = a.pairs
+        self.assertEqual(p.template._id, '1')
+        self.assertIs(p.template, t)
+        self.assertEqual(p.target._id, 'A')
 
     def test_template_ref_db_handler(self):
         """Test _TemplateRefDBHandler"""
@@ -419,7 +428,8 @@ _ma_template_details.target_asym_id
 _ma_template_details.template_label_asym_id
 _ma_template_details.template_label_entity_id
 _ma_template_details.template_model_num
-1 1 'reference database' polymer 1 1 A A 1 1
+_ma_template_details.template_auth_asym_id
+1 1 'reference database' polymer 1 1 A A 1 1 Z
 #
 loop_
 _ma_model_list.ordinal_id
@@ -467,7 +477,8 @@ _ma_template_details.target_asym_id
 _ma_template_details.template_label_asym_id
 _ma_template_details.template_label_entity_id
 _ma_template_details.template_model_num
-1 1 'reference database' polymer 1 1 A A 1 1
+_ma_template_details.template_auth_asym_id
+1 1 'reference database' polymer 1 1 A A 1 1 Z
 #
 loop_
 _ma_data_group.ordinal_id
